@@ -1,3 +1,6 @@
+# coding=utf-8
+from datetime import datetime
+
 # parser.py - Module for parsing whois response data
 # Copyright (c) 2008 Andrey Petrov
 #
@@ -5,94 +8,136 @@
 # the MIT license: http://www.opensource.org/licenses/mit-license.php
 
 import re
-from datetime import datetime
-   
+try:
+    import dateutil.parser as dp
+    from time_zones import tz_data
+    DATEUTIL = True
+except ImportError:
+    DATEUTIL = False
+
+KNOWN_FORMATS = [
+    '%d-%b-%Y', 				# 02-jan-2000
+    '%Y-%m-%d', 				# 2000-01-02
+    '%d.%m.%Y', 				# 2.1.2000
+    '%Y.%m.%d',                 # 2000.01.02
+    '%Y/%m/%d',                 # 2000/01/02
+    '%d/%m/%Y',                 # 02/01/2013
+    '%Y. %m. %d.',              # 2000. 01. 02.
+    '%Y.%m.%d %H:%M:%S',        # 2014.03.08 10:28:24
+    '%d-%b-%Y %H:%M:%S %Z',		# 24-Jul-2009 13:20:03 UTC
+    '%a %b %d %H:%M:%S %Z %Y',  # Tue Jun 21 23:59:59 GMT 2011
+    '%Y-%m-%dT%H:%M:%SZ',       # 2007-01-26T19:10:31Z
+    '%Y-%m-%dT%H:%M:%S%z',      # 2013-12-06T08:17:22-0800
+    '%Y-%m-%d %H:%M:%SZ',       # 2000-08-22 18:55:20Z
+    '%Y-%m-%d %H:%M:%S',        # 2000-08-22 18:55:20
+    '%d %b %Y %H:%M:%S',        # 08 Apr 2013 05:44:00
+    '%d/%m/%Y %H:%M:%S',     # 23/04/2015 12:00:07 EEST
+    '%d/%m/%Y %H:%M:%S %Z',     # 23/04/2015 12:00:07 EEST
+    '%d/%m/%Y %H:%M:%S.%f %Z',  # 23/04/2015 12:00:07.619546 EEST
+]
+
 
 class PywhoisError(Exception):
     pass
 
 
-def cast_date(s):
-    """Convert any date string found in WHOIS to a datetime object.
-    """
-    known_formats = [
-        '%d-%b-%Y', 				# 02-jan-2000
-        '%Y-%m-%d', 				# 2000-01-02
-        '%d.%m.%Y', 				# 2.1.2000
-        '%Y.%m.%d',                 # 2000.01.02
-        '%Y/%m/%d',                 # 2000/01/02
-        '%d-%b-%Y %H:%M:%S %Z',		# 24-Jul-2009 13:20:03 UTC
-        '%a %b %d %H:%M:%S %Z %Y',  # Tue Jun 21 23:59:59 GMT 2011
-        '%Y-%m-%dT%H:%M:%SZ',       # 2007-01-26T19:10:31Z
-    ]
-
-    for known_format in known_formats:
+def datetime_parse(s):
+    for known_format in KNOWN_FORMATS:
         try:
-            return datetime.strptime(s.strip(), known_format)
+            s = datetime.strptime(s.strip(), known_format)
+            break
         except ValueError as e:
-            pass # Wrong format, keep trying
+            pass  # Wrong format, keep trying
     return s
 
+def cast_date(s, dayfirst=False, yearfirst=False):
+    """Convert any date string found in WHOIS to a datetime object.
+    """
+    if DATEUTIL:
+        try:
+            return dp.parse(
+                s.strip(),
+                tzinfos=tz_data,
+                dayfirst=dayfirst,
+                yearfirst=yearfirst
+            ).replace(tzinfo=None)
+        except Exception:
+            return datetime_parse(s)
+    else:
+        return datetime_parse(s)
 
-class WhoisEntry(object):
+
+class WhoisEntry(dict):
     """Base class for parsing a Whois entries.
     """
     # regular expressions to extract domain data from whois profile
     # child classes will override this
     _regex = {
-        'domain_name':      'Domain Name:\s?(.+)',
-        'registrar':        'Registrar:\s?(.+)',
-        'whois_server':     'Whois Server:\s?(.+)',
-        'referral_url':     'Referral URL:\s?(.+)', # http url of whois_server
-        'updated_date':     'Updated Date:\s?(.+)',
-        'creation_date':    'Creation Date:\s?(.+)',
-        'expiration_date':  'Expiration Date:\s?(.+)',
-        'name_servers':     'Name Server:\s?(.+)', # list of name servers
-        'status':           'Status:\s?(.+)', # list of statuses
-        'emails':           '[\w.-]+@[\w.-]+\.[\w]{2,4}', # list of email addresses
+        'domain_name':          'Domain Name:\s?(.+)',
+        'registrar':            'Registrar:\s?(.+)',
+        'whois_server':         'Whois Server:\s?(.+)',
+        'referral_url':         'Referral URL:\s?(.+)',  # http url of whois_server
+        'updated_date':         'Updated Date:\s?(.+)',
+        'creation_date':        'Creation Date:\s?(.+)',
+        'expiration_date':      'Expir\w+ Date:\s?(.+)',
+        'name_servers':         'Name Server:\s?(.+)',  # list of name servers
+        'status':               'Status:\s?(.+)',  # list of statuses
+        'emails':               '[\w.-]+@[\w.-]+\.[\w]{2,4}',  # list of email s
+        'dnssec':               'dnssec:\s*([\S]+)',
+        'name':                 'Registrant Name:\s*(.+)',
+        'org':                  'Registrant\s*Organization:\s*(.+)',
+        'address':              'Registrant Street:\s*(.+)',
+        'city':                 'Registrant City:\s*(.+)',
+        'state':                'Registrant State/Province:\s*(.+)',
+        'zipcode':              'Registrant Postal Code:\s*(.+)',
+        'country':              'Registrant Country:\s*(.+)',
     }
+    dayfirst = False
+    yearfirst = False
 
     def __init__(self, domain, text, regex=None):
-        self.domain = domain
-        self.text = text
-        if regex is not None:
-            self._regex = regex
+        if 'This TLD has no whois server, but you can access the whois database at' in text:
+            raise PywhoisError(text)
+        else:
+            self.domain = domain
+            self.text = text
+            if regex is not None:
+                self._regex = regex
+            self.parse()
 
-
-    def __getattr__(self, attr):
+    def parse(self):
         """The first time an attribute is called it will be calculated here.
         The attribute is then set to be accessed directly by subsequent calls.
         """
-        whois_regex = self._regex.get(attr)
-        if whois_regex:
-            values = []
-            for value in re.findall(whois_regex, self.text, re.IGNORECASE):
-                if isinstance(value, basestring):
-                    # try casting to date format
-                    value = cast_date(value.strip())
-                values.append(value)
-            if len(values) == 1:
-                values = values[0]
-            setattr(self, attr, values)
-            return getattr(self, attr)
-        else:
-            raise KeyError('Unknown attribute: %s' % attr)
+        for attr, regex in self._regex.items():
+            if regex:
+                values = []
+                for value in re.findall(regex, self.text, re.IGNORECASE):
+                    if isinstance(value, basestring):
+                        # try casting to date format
+                        value = cast_date(value.strip(),
+                                          dayfirst=self.dayfirst,
+                                          yearfirst=self.yearfirst)
+                    if value and value not in values:
+                        # avoid duplicates
+                        values.append(value)
+                if len(values) == 1:
+                    values = values[0]
+                elif not values:
+                    values = None
 
-    def __str__(self):
-        """Print all whois properties of domain
-        """
-        return '\n'.join('%s: %s' % (attr, str(getattr(self, attr))) for attr in self.attrs())
+                self[attr] = values
 
 
-    def attrs(self):
-        """Return list of attributes that can be extracted for this domain
-        """
-        return sorted(self._regex.keys())
+    def __setitem__(self, name, value):
+        super(WhoisEntry, self).__setitem__(name, value)
+        setattr(self, name, value)
 
 
     @staticmethod
     def load(domain, text):
-        """Given whois output in ``text``, return an instance of ``WhoisEntry`` that represents its parsed contents.
+        """Given whois output in ``text``, return an instance of ``WhoisEntry``
+        that represents its parsed contents.
         """
         if text.strip() == 'No whois server is known for this kind of object.':
             raise PywhoisError(text)
@@ -104,28 +149,51 @@ class WhoisEntry(object):
         elif domain.endswith('.org'):
             return WhoisOrg(domain, text)
         elif domain.endswith('.name'):
-        	return WhoisName(domain, text)
+            return WhoisName(domain, text)
         elif domain.endswith('.me'):
-        	return WhoisMe(domain, text)
+            return WhoisMe(domain, text)
+        elif domain.endswith('.au'):
+            return WhoisAU(domain, text)
         elif domain.endswith('.ru'):
             return WhoisRu(domain, text)
         elif domain.endswith('.us'):
-        	return WhoisUs(domain, text)
+            return WhoisUs(domain, text)
         elif domain.endswith('.uk'):
-        	return WhoisUk(domain, text)
+            return WhoisUk(domain, text)
         elif domain.endswith('.fr'):
             return WhoisFr(domain, text)
+        elif domain.endswith('.nl'):
+            return WhoisNl(domain, text)
         elif domain.endswith('.fi'):
-        	return WhoisFi(domain, text)
+            return WhoisFi(domain, text)
         elif domain.endswith('.jp'):
             return WhoisJp(domain, text)
         elif domain.endswith('.pl'):
             return WhoisPl(domain, text)
+        elif domain.endswith('.br'):
+            return WhoisBr(domain, text)
+        elif domain.endswith('.eu'):
+            return WhoisEu(domain, text)
+        elif domain.endswith('.kr'):
+            return WhoisKr(domain, text)
+        elif domain.endswith('.pt'):
+            return WhoisPt(domain, text)
+        elif domain.endswith('.bg'):
+            return WhoisBg(domain, text)
+        elif domain.endswith('.de'):
+            return WhoisDe(domain, text)
+        elif domain.endswith('.ca'):
+            return WhoisCa(domain, text)
+        elif domain.endswith('.be'):
+            return WhoisBe(domain, text)
+        elif domain.endswith('.рф'):
+            return WhoisRf(domain, text)
+        elif domain.endswith('.info'):
+            return WhoisInfo(domain, text)
         else:
             return WhoisEntry(domain, text)
 
 
-
 class WhoisCom(WhoisEntry):
     """Whois parser for .com domains
     """
@@ -133,7 +201,7 @@ class WhoisCom(WhoisEntry):
         if 'No match for "' in text:
             raise PywhoisError(text)
         else:
-            WhoisEntry.__init__(self, domain, text) 
+            WhoisEntry.__init__(self, domain, text)
 
 
 class WhoisNet(WhoisEntry):
@@ -143,17 +211,30 @@ class WhoisNet(WhoisEntry):
         if 'No match for "' in text:
             raise PywhoisError(text)
         else:
-            WhoisEntry.__init__(self, domain, text) 
+            WhoisEntry.__init__(self, domain, text)
 
 
 class WhoisOrg(WhoisEntry):
     """Whois parser for .org domains
     """
+    regex = {
+        'domain_name':      'Domain Name:\s?(.+)',
+        'registrar':        'Registrar:\s?(.+)',
+        'whois_server':     'Whois Server:\s?(.+)', # empty usually
+        'referral_url':     'Referral URL:\s?(.+)', # http url of whois_server: empty usually
+        'updated_date':     'Updated Date:\s?(.+)',
+        'creation_date':    'Creation Date:\s?(.+)',
+        'expiration_date':  'Registry Expiry Date:\s?(.+)',
+        'name_servers':     'Name Server:\s?(.+)', # list of name servers
+        'status':           'Status:\s?(.+)', # list of statuses
+        'emails':           '[\w.-]+@[\w.-]+\.[\w]{2,4}', # list of email addresses
+    }
+    
     def __init__(self, domain, text):
         if text.strip() == 'NOT FOUND':
             raise PywhoisError(text)
         else:
-            WhoisEntry.__init__(self, domain, text) 
+            WhoisEntry.__init__(self, domain, text)
 
 
 class WhoisRu(WhoisEntry):
@@ -167,6 +248,7 @@ class WhoisRu(WhoisEntry):
         'name_servers': 'nserver:\s*(.+)',  # list of name servers
         'status': 'state:\s*(.+)',  # list of statuses
         'emails': '[\w.-]+@[\w.-]+\.[\w]{2,4}',  # list of email addresses
+        'org': 'org:\s*(.+)'
     }
 
     def __init__(self, domain, text):
@@ -176,11 +258,38 @@ class WhoisRu(WhoisEntry):
             WhoisEntry.__init__(self, domain, text, self.regex)
 
 
+class WhoisNl(WhoisEntry):
+    """Whois parser for .nl domains
+    """
+    regex = {
+        'name': None,
+        'address': None,
+        'zip_code': None,
+        'city': None,
+        'country': None
+    }
+
+    def __init__(self, domain, text):
+        if text.endswith('is free'):
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+        match = re.compile('Registrar:(.*?)DNSSEC', re.DOTALL).search(text)
+        if match:
+            lines = match.groups()[0].strip().splitlines()
+            self.name = lines[0]
+            self.address = lines[1]
+            if len(lines) == 4:
+                self.zip_code, _, self.city = lines[2].partition(' ')
+            self.country = lines[-1]
+                
+
 class WhoisName(WhoisEntry):
     """Whois parser for .name domains
     """
     regex = {
-    	'domain_name_id':  'Domain Name ID:\s*(.+)',
+        'domain_name_id':  'Domain Name ID:\s*(.+)',
         'domain_name':     'Domain Name:\s*(.+)',
         'registrar_id':    'Sponsoring Registrar ID:\s*(.+)',
         'registrar':       'Sponsoring Registrar:\s*(.+)',
@@ -194,23 +303,24 @@ class WhoisName(WhoisEntry):
         'name_server_ids': 'Name Server ID:\s*(.+)',  # list of name server ids
         'name_servers':    'Name Server:\s*(.+)',  # list of name servers
         'status':          'Domain Status:\s*(.+)',  # list of statuses
-	}
+    }
+
     def __init__(self, domain, text):
-        if 'No match.' in text:
+        if 'No match for ' in text:
             raise PywhoisError(text)
         else:
-            WhoisEntry.__init__(self, domain, text, self.regex) 
-    
-        
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
 class WhoisUs(WhoisEntry):
     """Whois parser for .us domains
     """
     regex = {
         'domain_name':                    'Domain Name:\s*(.+)',
-    	'domain__id':                     'Domain ID:\s*(.+)',
+        'domain__id':                     'Domain ID:\s*(.+)',
         'registrar':                      'Sponsoring Registrar:\s*(.+)',
         'registrar_id':                   'Sponsoring Registrar IANA ID:\s*(.+)',
-        'registrar_url':                  'Registrar URL \(registration services\):\s*(.+)',        
+        'registrar_url':                  'Registrar URL \(registration services\):\s*(.+)',
         'status':                         'Domain Status:\s*(.+)',  # list of statuses
         'registrant_id':                  'Registrant ID:\s*(.+)',
         'registrant_name':                'Registrant Name:\s*(.+)',
@@ -270,39 +380,56 @@ class WhoisUs(WhoisEntry):
         'creation_date':                  'Domain Registration Date:\s*(.+)',
         'expiration_date':                'Domain Expiration Date:\s*(.+)',
         'updated_date':                   'Domain Last Updated Date:\s*(.+)',
-	}
+    }
+
     def __init__(self, domain, text):
         if 'Not found:' in text:
             raise PywhoisError(text)
         else:
             WhoisEntry.__init__(self, domain, text, self.regex)
-       
+
 
 class WhoisPl(WhoisEntry):
-   """Whois parser for .uk domains
-   """
-   regex = {
-       'domain_name':                    'DOMAIN NAME:\s*(.+)\n',
-       'registrar':                      'REGISTRAR:\n\s*(.+)',
-       'registrar_url':                  'URL:\s*(.+)',        # not available
-       'status':                         'Registration status:\n\s*(.+)',  # not available
-       'registrant_name':                'Registrant:\n\s*(.+)',   # not available
-       'creation_date':                  'created:\s*(.+)\n',
-       'expiration_date':                'renewal date:\s*(.+)',
-       'updated_date':                   'last modified:\s*(.+)\n',
-   }
-   def __init__(self, domain, text):
-       if 'Not found:' in text:
-           raise PywhoisError(text)
-       else:
-           WhoisEntry.__init__(self, domain, text, self.regex)
- 
-    
+    """Whois parser for .pl domains
+    """
+    regex = {
+        'domain_name':                    'DOMAIN NAME:\s*(.+)\n',
+        'registrar':                      'REGISTRAR:\n\s*(.+)',
+        'registrar_url':                  'URL:\s*(.+)',        # not available
+        'status':                         'Registration status:\n\s*(.+)',  # not available
+        'registrant_name':                'Registrant:\n\s*(.+)',   # not available
+        'creation_date':                  'created:\s*(.+)\n',
+        'expiration_date':                'renewal date:\s*(.+)',
+        'updated_date':                   'last modified:\s*(.+)\n',
+    }
+
+    def __init__(self, domain, text):
+        if 'No information available about domain name' in text:
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+class WhoisCa(WhoisEntry):
+    """Whois parser for .ca domains
+    """
+    regex = {
+        'registrant_name':                'Name:\s*(.+)',
+        'registrant_number':              'Number:\s*(.+)\n',
+    }
+
+    def __init__(self, domain, text):
+        if 'Domain status:         available' in text:
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
 class WhoisMe(WhoisEntry):
     """Whois parser for .me domains
     """
     regex = {
-    	'domain_id':                   'Domain ID:(.+)',
+        'domain_id':                   'Domain ID:(.+)',
         'domain_name':                 'Domain Name:(.+)',
         'creation_date':               'Domain Create Date:(.+)',
         'updated_date':                'Domain Last Updated Date:(.+)',
@@ -363,12 +490,13 @@ class WhoisMe(WhoisEntry):
         'tech_fax_ext':                'Tech FAX Ext\.:(.+)',
         'tech_email':                  'Tech E-mail:(.+)',
         'name_servers':                'Nameservers:(.+)',  # list of name servers
-	}
+    }
+
     def __init__(self, domain, text):
         if 'NOT FOUND' in text:
             raise PywhoisError(text)
         else:
-            WhoisEntry.__init__(self, domain, text, self.regex) 
+            WhoisEntry.__init__(self, domain, text, self.regex)
 
 
 class WhoisUk(WhoisEntry):
@@ -384,9 +512,10 @@ class WhoisUk(WhoisEntry):
         'expiration_date':                'Expiry date:\s*(.+)',
         'updated_date':                   'Last updated:\s*(.+)',
         'name_servers':                   'Name servers:\s*(.+)',
-	}
+    }
+
     def __init__(self, domain, text):
-        if 'Not found:' in text:
+        if 'No match for ' in text:
             raise PywhoisError(text)
         else:
             WhoisEntry.__init__(self, domain, text, self.regex)
@@ -407,7 +536,7 @@ class WhoisFr(WhoisEntry):
     }
 
     def __init__(self, domain, text):
-        if text.strip() == 'No entries found':
+        if 'No entries found' in text:
             raise PywhoisError(text)
         else:
             WhoisEntry.__init__(self, domain, text, self.regex)
@@ -418,17 +547,18 @@ class WhoisFi(WhoisEntry):
     """
     regex = {
         'domain_name':                    'domain:\s*([\S]+)',
-        'registrant_name':                'descr:\s*([\S\ ]+)',
-        'registrant_address':             'address:\s*([\S\ ]+)',
-        'registrant_phone':               'phone:\s*([\S\ ]+)',
+        'name':                           'descr:\s*([\S\ ]+)',
+        'address':                        'address:\s*([\S\ ]+)',
+        'phone':                          'phone:\s*([\S\ ]+)',
         'status':                         'status:\s*([\S]+)',  # list of statuses
         'creation_date':                  'created:\s*([\S]+)',
         'updated_date':                   'modified:\s*([\S]+)',
         'expiration_date':                'expires:\s*([\S]+)',
         'name_servers':                   'nserver:\s*([\S]+) \[\S+\]',  # list of name servers
         'name_server_statuses':           'nserver:\s*([\S]+) \[(\S+)\]',  # list of name servers and statuses
-        'dnssec':                   'dnssec:\s*([\S]+)',
-	}
+        'dnssec':                         'dnssec:\s*([\S]+)',
+    }
+
     def __init__(self, domain, text):
         if 'Domain not ' in text:
             raise PywhoisError(text)
@@ -448,8 +578,225 @@ class WhoisJp(WhoisEntry):
         'status': '\[State\]\s*(.+)',  # list of statuses
     }
 
+    def __init__(self, domain, text):
+        if 'No match!!' in text:
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+class WhoisAU(WhoisEntry):
+    """Whois parser for .au domains
+    """
+    regex = {
+        'domain_name':                    'Domain Name:\s*(.+)\n',
+        'last_modified':			      'Last Modified:\s*(.+)\n',
+        'registrar':                      'Registrar Name:\s*(.+)\n',
+        'status':                         'Status:\s*(.+)',
+        'registrant_name':                'Registrant:\s*(.+)',
+        'name_servers':                   'Name Server:\s*(.+)',
+    }
+
+    def __init__(self, domain, text):
+        if text.strip() == 'No Data Found':
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+class WhoisEu(WhoisEntry):
+    """Whois parser for .eu domains
+    """
+    regex = {
+        'domain_name': r'Domain:\s*([^\n\r]+)',
+        'tech_name': r'Technical:\s*Name:\s*([^\n\r]+)',
+        'tech_org': r'Technical:\s*Name:\s*[^\n\r]+\s*Organisation:\s*([^\n\r]+)',
+        'tech_phone': r'Technical:\s*Name:\s*[^\n\r]+\s*Organisation:\s*[^\n\r]+\s*Language:\s*[^\n\r]+\s*Phone:\s*([^\n\r]+)',
+        'tech_fax': r'Technical:\s*Name:\s*[^\n\r]+\s*Organisation:\s*[^\n\r]+\s*Language:\s*[^\n\r]+\s*Phone:\s*[^\n\r]+\s*Fax:\s*([^\n\r]+)',
+        'tech_email': r'Technical:\s*Name:\s*[^\n\r]+\s*Organisation:\s*[^\n\r]+\s*Language:\s*[^\n\r]+\s*Phone:\s*[^\n\r]+\s*Fax:\s*[^\n\r]+\s*Email:\s*([^\n\r]+)',
+        'registrar': r'Registrar:\s*Name:\s*([^\n\r]+)',
+        'name_servers': r'Name servers:\s*([^\n\r]+)\s*([^\n\r]*)',  # list of name servers
+    }
+
+    def __init__(self, domain, text):
+        if text.strip() == 'Status: AVAILABLE':
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+class WhoisBr(WhoisEntry):
+    """Whois parser for .br domains
+    """
+    regex = {
+        'domain':                        'domain:\s*(.+)\n',
+        'owner':                         'owner:\s*([\S ]+)',
+        'ownerid':                       'ownerid:\s*(.+)',
+        'country':                       'country:\s*(.+)',
+        'owner_c':                       'owner-c:\s*(.+)',
+        'admin_c':                       'admin-c:\s*(.+)',
+        'tech_c':                        'tech-c:\s*(.+)',
+        'billing_c':                     'billing-c:\s*(.+)',
+        'nserver':                       'nserver:\s*(.+)',
+        'nsstat':                        'nsstat:\s*(.+)',
+        'nslastaa':                      'nslastaa:\s*(.+)',
+        'saci':                          'saci:\s*(.+)',
+        'created':                       'created:\s*(.+)',
+        'expires':                       'expires:\s*(.+)',
+        'changed':                       'changed:\s*(.+)',
+        'status':                        'status:\s*(.+)',
+        'nic_hdl_br':                    'nic-hdl-br:\s*(.+)',
+        'person':                        'person:\s*([\S ]+)',
+        'email':                         'e-mail:\s*(.+)',
+    }
+
+    def __init__(self, domain, text):
+
+        if 'Not found:' in text:
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+class WhoisKr(WhoisEntry):
+    """Whois parser for .kr domains
+    """
+    regex = {
+        'domain_name': 'Domain Name\s*:\s*(.+)',
+        'registrant_org': 'Registrant\s*:\s*(.+)',
+        'registrant_address': 'Registrant Address\s*:\s*(.+)',
+        'registrant_zip': 'Registrant Zip Code\s*:\s*(.+)',
+        'admin_name': 'Administrative Contact\(AC\)\s*:\s*(.+)',
+        'admin_email': 'AC E-Mail\s*:\s*(.+)',
+        'admin_phone': 'AC Phone Number\s*:\s*(.+)',
+        'creation_date': 'Registered Date\s*:\s*(.+)',
+        'updated_date':  'Last updated Date\s*:\s*(.+)',
+        'expiration_date':  'Expiration Date\s*:\s*(.+)',
+        'registrar':  'Authorized Agency\s*:\s*(.+)',
+        'name_servers': 'Host Name\s*:\s*(.+)',  # list of name servers
+    }
+
+    def __init__(self, domain, text):
+        if text.endswith(' no match'):
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+class WhoisPt(WhoisEntry):
+    """Whois parser for .pt domains
+    """
+    regex = {
+        'domain_name': 'domain name:\s*(.+)',
+        'creation_date': 'creation date \(dd\/mm\/yyyy\):\s*(.+)',
+        'expiration_date': 'expiration date \(dd\/mm\/yyyy\):\s*(.+)',
+        'name_servers': '\tNS\t(.+).',  # list of name servers
+        'status': 'status:\s*(.+)',  # list of statuses
+        'emails': '[\w.-]+@[\w.-]+\.[\w]{2,4}',  # list of email addresses
+    }
+
     def __init__(self, domain, text):
         if text.strip() == 'No entries found':
             raise PywhoisError(text)
         else:
             WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+class WhoisBg(WhoisEntry):
+    """Whois parser for .bg domains"""
+
+    regex = {
+        'expiration_date': 'expires at:\s*(.+)',
+    }
+
+    dayfirst = True
+
+    def __init__(self, domain, text):
+        if 'does not exist in database!' in text:
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+class WhoisRf(WhoisEntry):
+    """Whois parser for .rf domains"""
+
+    regex = {
+        'expiration_date': 'free-date:\s*(.+)',
+    }
+
+    def __init__(self, domain, text):
+        if text.strip() == 'No entries found':
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+class WhoisDe(WhoisEntry):
+    """Whois parser for .de domains"""
+
+    regex = {
+        'name': 'name:\s*(.+)',
+        'org': 'Organisation:\s*(.+)',
+        'address': 'Address:\s*(.+)',
+        'zipcode': 'PostalCode:\s*(.+)',
+        'city': 'City:\s*(.+)',
+        'country_code': 'CountryCode:\s*(.+)',
+        'phone': 'Phone:\s*(.+)',
+        'fax': 'Fax:\s*(.+)'
+    }
+
+    def __init__(self, domain, text):
+        if 'Status: free' in text:
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+class WhoisBe(WhoisEntry):
+    """Whois parser for .be domains"""
+
+    regex = {
+        'name': 'Name:\s*(.+)',
+        'org': 'Organisation:\s*(.+)',
+        'phone': 'Phone:\s*(.+)',
+        'fax': 'Fax:\s*(.+)',
+        'email': 'Email:\s*(.+)',
+    }
+
+    def __init__(self, domain, text):
+        if 'Status: AVAILABLE' in text:
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
+
+
+
+class WhoisInfo(WhoisEntry):
+    """Whois parser for .info domains
+    """
+    regex = {
+        'domain_name':      'Domain Name:\s?(.+)',
+        'registrar':        'Registrar:\s?(.+)',
+        'whois_server':     'Whois Server:\s?(.+)', # empty usually
+        'referral_url':     'Referral URL:\s?(.+)', # http url of whois_server: empty usually
+        'updated_date':     'Updated Date:\s?(.+)',
+        'creation_date':    'Creation Date:\s?(.+)',
+        'expiration_date':  'Registry Expiry Date:\s?(.+)',
+        'name_servers':     'Name Server:\s?(.+)', # list of name servers
+        'status':           'Status:\s?(.+)', # list of statuses
+        'emails':           '[\w.-]+@[\w.-]+\.[\w]{2,4}', # list of email addresses
+        'name':             'Registrant Name:\s*(.+)',
+        'org':              'Registrant Organization:\s*(.+)',
+        'address':          'Registrant Street:\s*(.+)',
+        'city':             'Registrant City:\s*(.+)',
+        'state':            'Registrant State/Province:\s*(.+)',
+        'zipcode':          'Registrant Postal Code:\s*(.+)',
+        'country':          'Registrant Country:\s*(.+)',
+    }
+
+    def __init__(self, domain, text):
+        if text.strip() == 'NOT FOUND':
+            raise PywhoisError(text)
+        else:
+            WhoisEntry.__init__(self, domain, text, self.regex)
